<?php
/*
Plugin Name: Data Source for Contact Form 7
Plugin URI: https://cf7-datasource.dwbooster.com/
Description: Data Source for Contact Form 7 plugin allows populating the "Contact Form 7" fields (text, email, URL, drop-down menu, ...) with data stored in external data sources, like a database, CSV file, users information, post data, taxonomies, and JSON objects.
Version: 1.0.18
Author: CodePeople.net
Author URI: https://cf7-datasource.dwbooster.com/about-us
License: GPL
*/

require_once dirname(__FILE__).'/banner.php';
$codepeople_promote_banner_plugins[ 'cf7-datasource' ] = array(
	'plugin_name' => 'Data Source for Contact Form 7',
	'plugin_url'  => 'https://wordpress.org/support/plugin/cf7-data-source/reviews/#new-post'
);

// Feedback system
require_once 'feedback/cp-feedback.php';
new CP_FEEDBACK(basename(dirname(__FILE__)), __FILE__, 'https://cf7-datasource.dwbooster.com/contact-us');

if(!class_exists('CF7_DATASOURCE'))
{
	class CF7_DATASOURCE
	{
		// Properties
		static public $plugin = 'cf7-datasource';
		static private $plugin_id = 639;
		static public $version = '1.0.18';
		static private $obj;

		private $debug = true;
		private $errors;
		private $cf7; 	 // CF7 id
		private $prefix; // Prefix for fields
		private $ds_manager;

		// Class methods
		public static function install($networkwide)
		{
			global $wpdb;
			$obj = new CF7_DATASOURCE;
			if (function_exists('is_multisite') && is_multisite())
			{
				if ($networkwide)
				{
					$old_blog = $wpdb->blogid;

					$blogids = $wpdb->get_col( "SELECT blog_id FROM $wpdb->blogs" );
					foreach ($blogids as $blog_id)
					{
						switch_to_blog($blog_id);
						$obj->validL();
						$this->_install();
					}
					switch_to_blog($old_blog);
					return;
				}
			}
			$obj->validL();
		} // End install

		public static function init()
		{
			// Create instance
			self::$obj = new CF7_DATASOURCE;

			// Check if it is calling data
			if(!empty($_REQUEST['cf7_recordset']))
			{
				$data = self::$obj->sanitize_deep($_REQUEST);
				self::$obj->get_data($data);
			}
			else self::$obj->deleteL();
		}

		/********************** INSTANCE METHODS AND PROPERTIES **********************/
		public function __construct()
		{
			$banner = dirname(__FILE__).'/banner.php';
			if(file_exists($banner)) require_once $banner;

			require_once dirname(__FILE__).'/datasources/cf7-datasource-manager.php';
			$this->ds_manager = new CF7_DATASOURCE_MANAGER($this);
			$this->errors = [];

			add_shortcode('cf7-recordset', array($this, 'cf7_recordset'));
			add_shortcode('cf7-link-field', array($this, 'cf7_link'));

			// Public form
			add_filter('wpcf7_form_elements', function($content){
				$cf7 = WPCF7_ContactForm::get_current();
				$this->generate_prefix($cf7->id);
				return do_shortcode($content);
			});

			// Load editor
			if(is_admin())
			{
				$path = dirname(__FILE__).'/admin/cf7-datasource-editor.php';
				if(file_exists($path)){ require_once $path; $cf7_datasource_editor = new CF7_DATASOURCE_EDITOR($this->ds_manager);}
			}

		} // End __construct

		/*************************** PUBLIC METHODS  ***************************/
		public function sanitize_deep($values)
		{
			$values = stripslashes_deep($values);
			if(is_string($values)) $values = sanitize_text_field($values);
			elseif(is_array($values))
			foreach($values as $key => $value) $values[$key] = $this->sanitize_deep($value);
			return $values;
		} // End sanitize_deep

		public function get_data($data)
		{
			$this->generate_prefix(isset($data['cf7']) ? @intval($data['cf7']) : 0);
			$recordset = sanitize_key($data['cf7_recordset']);
			$attrs = get_transient($this->prefix.$recordset);

			if($attrs != false)
			{
				$attrs = $this->prefill_recordset($attrs);
				array_walk($attrs, function(&$v,$i){$v = html_entity_decode($v);});
				$this->debug = !empty($attrs['debug']) ? true : false;
				$records = $this->ds_manager->ds($attrs, $data);
			} else $this->error_log('The recordset settings return false');

			$result = new stdClass();
			$errors = $this->error_message();

			if(!empty($errors))  $result->error = $errors;
			else $result->data = $records;

			print(json_encode($result));
			exit;
		} // End get_data

		public function cf7_recordset($attrs, $content='')
		{
			$attrs = $this->prefill_recordset($attrs);
			$this->debug = isset($attrs['debug']) ? @intval($attrs['debug']) : 0;
			$attrs['id'] = trim($attrs['id']);
			if(!empty($attrs['id']))
			{
				delete_transient($this->prefix.$attrs['id']);
				if(set_transient($this->prefix.$attrs['id'], $attrs, 24*60*60))
				{
					// encode script output code
					$this->js_recordset($attrs);
				}
				else $this->error_log('The data source field '.$attrs['id'].' could not be registered');
			}
			else $this->error_log('Field id is required');

			return '';
		} // End cf7_recordset

		public function cf7_link($attrs, $content='')
		{
			$attrs = $this->prefill_link($attrs);
			$this->debug = isset($attrs['debug']) ? @intval($attrs['debug']) : 0;

			$attrs['recordset'] = trim($attrs['recordset']);
			$attrs['field'] = trim($attrs['field']);

			if(!empty($attrs['recordset']) && !empty($attrs['field']))
			{
				// encode script output code
				$this->js_link($attrs);
			}
			else $this->error_log('The field and recordset attributes are required');
			return '';
		} // End cf7_link

		public function error_log($mssg)
		{
			$this->errors[] = $mssg;
			if(
				$this->debug &&
				!empty($mssg)
			)
			{
				error_log('Data Source for Contact Form 7: ' .$mssg);
			}
		} // End error_log

		/*************************** PRIVATE METHODS ***************************/

		private function error_message()
		{
			return implode(' ', $this->errors);
		} // End error_message

		private function prefill_recordset($settings)
		{
			return shortcode_atts(
				$this->ds_manager->default_datasource_attributes(),
				$settings
			);
		} // End prefill_recordset

		private function prefill_link($settings)
		{
			return shortcode_atts(
				array(
					'recordset' => '', // Recordset field name
					'field'		=> '', // Namem of the field to populate
					'value' 	=> '', // Column name for values
					'text'		=> '', // Column name for texts
					'condition' => ''  // Condition for filtering the rows
				),
				$settings
			);
		} // End prefill_link

		private function generate_prefix($cf7)
		{
			$this->cf7 = $cf7;
			$this->prefix = 'cf7_'.$this->cf7.'_';
		} // End generate_prefix

		private function js()
		{
			wp_enqueue_script('jquery');
			wp_enqueue_script('cf7_datasource_script', plugins_url('/assets/script.js', __FILE__), array('jquery'), self::$version);
		} // End js

		private function js_recordset($attrs)
		{
			$this->js();

			// Extract variables
			$str = implode(
				' ',
				array(
					isset($attrs['query']) ? $attrs['query'] : '',
					isset($attrs['condition']) ? $attrs['condition'] : '',
					isset($attrs['variable']) ? $attrs['variable'] : ''
				)
			);
			$js_settings = $this->extract_vars($str);
			$js_settings['cf7']	= $this->cf7;

			wp_add_inline_script('cf7_datasource_script', 'cf7_datasource_register_recordset("'.esc_js($attrs['id']).'", '.json_encode($js_settings).')', 'after');
		} // End js_recordset

		private function js_link($attrs)
		{
			$this->js();
			// Extract variables
			$str = $attrs['condition'];
			$js_settings = $this->extract_vars($str);
			$attrs = array_merge($attrs, $js_settings);
			wp_add_inline_script('cf7_datasource_script', 'cf7_datasource_register_link('.json_encode($attrs).')', 'after');
		} // End js_link

		private function extract_vars($str)
		{
			$variables = array(
				'fields' 	=> array(),
				'variables' => array()
			);

			if(preg_match_all('/\{field\.([^\}]+)\}/i', $str, $m_fields)) $variables['fields'] = $m_fields[1];
			if(preg_match_all('/\{var\.([^\}]+)\}/i', $str, $m_vars)) $variables['variables'] = $m_vars[1];

			return $variables;
		} // End extract_vars

		public function validL()
		{
			$l = get_option( self::$plugin.'_LICENSE', '');
			if ($l)
			{
				$v = "https://wordpress.dwbooster.com/licensesystem/code/v.php?id=".self::$plugin_id."&l=".$l."&w=".$_SERVER["HTTP_HOST"]."&a=info";
				try {
					$contents = wp_remote_get( $v );
					if(strpos($contents["body"],'not valid')){ update_option( self::$plugin.'_LICENSE', '' ); }
					else return true;
				} catch (Exception $e) {}
			}
			return false;
		} // End validL

		public function deleteL()
		{
			$key = self::$plugin.'_fixlk';
			$val = '';
			if(!empty($_GET[$key])) $val = sanitize_text_field($_GET[$key]);
			elseif(!empty($_POST[$key])) $val = sanitize_text_field($_POST[$key]);

			if(!empty($val) && $val == get_option( self::$plugin.'_LICENSE',''))
			{
				update_option(self::$plugin.'_LICENSE', '' );
				$key = self::$plugin.'_fixf';
				$val = '';
				if(!empty($_GET[$key])) $val = sanitize_text_field($_GET[$key]);
				elseif(!empty($_POST[$key])) $val = sanitize_text_field($_POST[$key]);

				if ($val == 'ok')
				{
				   @unlink( dirname( __FILE__ ) . '/datasources/cf7-datasource-advanced.php' );
				}
				echo 'Ok, LK fixed.';
				exit;
			}
		} // End deleteL
	} // End class CF7_DATASOURCE

	add_action('plugins_loaded', array('CF7_DATASOURCE', 'init'), 11);
	register_activation_hook(__FILE__, array('CF7_DATASOURCE', 'install'));
}