<?php
require_once dirname(__FILE__).'/cf7-datasource-base.php';
if(!class_exists('CF7_DATASOURCE_BASIC'))
{
	class CF7_DATASOURCE_BASIC extends CF7_DATASOURCE_BASE
	{
		public function __construct($manager)
		{
			parent::__construct($manager);
		} // End __construct

		public function default_attributes()
		{
			$attributes = array(
				/* POST */

				'attributes' => 'ID', // Attributes to get separated by comma, like post_title, post_author
				'condition'  => '', // Condition for filtering like post_status='publish' AND post_author=1

				/* TAXONOMY */

				'taxonomy'	=> '', // Taxonomy name. Ex. category
				// 'attributes' => 'name',
									/* attributes is commented because this attribute was included above.
									The attributes must be separated by comma, like term_id, name, slug */
				// 'condition'  => '',
									/* condition is commented because this attribute was included above.
									Ex. term_id = 1 OR slug='uncategorized' */

				/* USER */

				// 'attributes' => 'ID',
									/* attributes is commented because this attribute was included above.
									The attributes must be separated by comma, like nicename, email, first_name */
				'logged'	=> 0, // To get the data corresponding to the logged user enter 1
				// 'condition'  => '',
									/* condition is commented because this attribute was included above.
									Ex. user_login='admin'
									If the logged attribute is set to 1 it takes precedence */

                /* DATABASE */

                'dns' 		=> '',
				'engine' 	=> 'mysql',
				'hostname' 	=> '',
				'database' 	=> '',
				'username' 	=> '',
				'password' 	=> '',

				'query'		=> '',
			);

			return array_merge(parent::default_attributes(),$attributes);
		} // End default_attributes

		public function ds_options()
		{
			return '<option value="user">'.esc_html(__('Users information', 'cf7-datasource')).'</option>
					<option value="post">'.esc_html(__('Posts data (posts, pages, products, custom post types)', 'cf7-datasource')).'</option>
					<option value="taxonomy">'.esc_html(__('Taxonomies (categories, posts tags, custom taxonomies)', 'cf7-datasource')).'</option>'.
                    '<option value="database">'.esc_html(__('Read data from database', 'cf7-datasource')).'</option>';
		} // End ds_options

		public function ds_settings()
		{
            global $wpdb;

            // cf7-users-datalist
            $users_datalist_options = '';

            $rows = $wpdb->get_results('SHOW COLUMNS FROM `'.$wpdb->users.'`');
            foreach($rows as $row)
                $users_datalist_options .= '<option value="'.esc_attr($row->Field).'" />';

            $rows = $wpdb->get_results('SELECT DISTINCT meta_key FROM `'.$wpdb->usermeta.'`');
            foreach($rows as $row)
                $users_datalist_options .= '<option value="'.esc_attr($row->meta_key).'" />';

            // cf7-posts-datalist
            $posts_datalist_options = '';

            $rows = $wpdb->get_results('SHOW COLUMNS FROM `'.$wpdb->posts.'`');
            foreach($rows as $row)
                $posts_datalist_options .= '<option value="'.esc_attr($row->Field).'" />';

            // cf7-taxonomies-list
            $taxonomies_datalist_options = '';
            $taxonomies = get_taxonomies(array(), 'names');
            foreach($taxonomies as $taxonomy)
                $taxonomies_datalist_options .= '<option value="'.esc_attr($taxonomy).'" />';

			return '<!-- Data source user -->
					<tr class="cf7-datasource-user">
						<th scope="row">'.esc_html( __( 'User data separated by comma', 'cf7-datasource' ) ).'</th>
						<td>
							<input aria-label="'.esc_attr__('User data', 'cf7-datasource').'" type="text" name="cf7-user-attributes" list="cf7-users-datalist" />
                            <datalist id="cf7-users-datalist">
                            '.$users_datalist_options.'
                            </datalist>
							<br><i>Ex. user_email,user_login,first_name</i>
						</td>
					</tr>
					<tr class="cf7-datasource-user">
						<th scope="row">'.esc_html( __( 'Get data of logged user', 'cf7-datasource' ) ).'</th>
						<td>
							<input aria-label="'.esc_attr__('For logged user', 'cf7-datasource').'" type="checkbox" name="cf7-user-logged" />
						</td>
					</tr>
					<tr class="cf7-datasource-user">
						<th scope="row">'.esc_html( __( 'Condition for filtering', 'cf7-datasource' ) ).'</th>
						<td>
							<input aria-label="'.esc_attr__('Condition for filtering', 'cf7-datasource').'" type="text" name="cf7-user-condition" />
							<br><i>Ex. user_email=\'{field.your-email}\'</i>
						</td>
					</tr>
					<tr class="cf7-datasource-user">
						<th scope="row"></th>
						<td style="text-align:right;">
							<a href="https://cf7-datasource.dwbooster.com/documentation#user-information" target="_blank" style="font-weight:bold;display:inline-block;padding:5px;border:2px solid #2271b1; color:#2271b1;text-decoration:none;">'.__('Data surce HELP', 'cf7-datasource').'</a>
						</td>
					</tr>

					<!-- Data source post -->
					<tr class="cf7-datasource-post">
						<th scope="row">'.esc_html( __( 'Post data separated by comma', 'cf7-datasource' ) ).'</th>
						<td>
							<input aria-label="'.esc_attr__('Post data', 'cf7-datasource').'" type="text" name="cf7-post-attributes" list="cf7-posts-datalist" />
                            <datalist id="cf7-posts-datalist">
                            '.$posts_datalist_options.'
                            </datalist>
							<br><i>Ex. ID,post_title,post_excerpt,post_status</i>
						</td>
					</tr>
					<tr class="cf7-datasource-post">
						<th scope="row">'.esc_html( __( 'Condition for filtering', 'cf7-datasource' ) ).'</th>
						<td>
							<input aria-label="'.esc_attr__('Condition for filtering', 'cf7-datasource').'" type="text" name="cf7-post-condition" />
							<br><i>Ex. post_status=\'publish\' AND post_author=1</i>
						</td>
					</tr>
					<tr class="cf7-datasource-post">
						<th scope="row"></th>
						<td style="text-align:right;">
							<a href="https://cf7-datasource.dwbooster.com/documentation#post-data" target="_blank" style="font-weight:bold;display:inline-block;padding:5px;border:2px solid #2271b1; color:#2271b1;text-decoration:none;">'.__('Data surce HELP', 'cf7-datasource').'</a>
						</td>
					</tr>

					<!-- Data source taxonomy -->
					<tr class="cf7-datasource-taxonomy">
						<th scope="row">'.esc_html( __( 'Taxonomy name', 'cf7-datasource' ) ).'</th>
						<td>
							<input aria-label="'.esc_attr__('Taxonomy name', 'cf7-datasource').'" type="text" name="cf7-taxonomy-name" list="cf7-taxonomies-list" />
                            <datalist id="cf7-taxonomies-list">
                            '.$taxonomies_datalist_options.'
                            </datalist>
							<br><i>Ex. category</i>
						</td>
					</tr>
					<tr class="cf7-datasource-taxonomy">
						<th scope="row">'.esc_html( __( 'Terms data separated by comma', 'cf7-datasource' ) ).'</th>
						<td>
							<input aria-label="'.esc_attr__('Terms data', 'cf7-datasource').'" type="text" name="cf7-taxonomy-attributes" />
							<br><i>Ex. term_id,name,slug</i>
						</td>
					</tr>
					<tr class="cf7-datasource-taxonomy">
						<th scope="row">'.esc_html( __('Condition for filtering', 'cf7-datasource' ) ).'</th>
						<td>
							<input aria-label="'.esc_attr__('Condition for filtering', 'cf7-datasource').'" type="text" name="cf7-taxonomy-condition" />
							<br><i>Ex. term_id=1 OR slug=\'uncategorized\'</i>
						</td>
					</tr>
					<tr class="cf7-datasource-taxonomy">
						<th scope="row"></th>
						<td style="text-align:right;">
							<a href="https://cf7-datasource.dwbooster.com/documentation#taxonomy" target="_blank" style="font-weight:bold;display:inline-block;padding:5px;border:2px solid #2271b1; color:#2271b1;text-decoration:none;">'.__('Data surce HELP', 'cf7-datasource').'</a>
						</td>
					</tr>

                    <!-- Data source database -->
					<tr class="cf7-datasource-database">
						<th scope="row">'.esc_html( __( 'Database connection', 'cf7-datasource' ) ).'</th>
						<td>
							<label><input aria-label="'.esc_attr__('Current website database', 'cf7-datasource').'" type="radio" name="cf7-database-connection" value="website" CHECKED /> '.__('Current website database', 'cf7-datasource').'</label>
							<label><input aria-label="'.esc_attr__('Connection components', 'cf7-datasource').'" type="radio" name="cf7-database-connection" value="components" /> '.__('Connection components', 'cf7-datasource').'</label>
							<label><input aria-label="'.esc_attr__('DNS', 'cf7-datasource').'" type="radio" name="cf7-database-connection" value="dns" /> '.__('DNS', 'cf7-datasource').'</label>
                            <a href="https://cf7-datasource.dwbooster.com/examples" target="_blank" style="float:right;">'.__('Get demos +', 'cf7-datasource').'</a>
						</td>
					</tr>
					<tr class="cf7-datasource-database cf7-datasource-dns cf7-datasource-components">
						<th scope="row"></th>
						<td>
							<b>'.esc_html( __('To read data from the current website database, leave the connection data blank', 'cf7-datasource' ) ).'</b>
						</td>
					</tr>
					<tr class="cf7-datasource-database cf7-datasource-dns">
						<th scope="row">'.esc_html( __('DNS', 'cf7-datasource' ) ).'</th>
						<td>
							<input aria-label="'.esc_attr__('DNS', 'cf7-datasource').'" type="text" name="cf7-database-dns" />
						</td>
					</tr>
					<tr class="cf7-datasource-database cf7-datasource-components">
						<th scope="row">'.esc_html( __('Engine', 'cf7-datasource' ) ).'</th>
						<td>
							<input aria-label="'.esc_attr__('Engine', 'cf7-datasource').'" type="text" name="cf7-database-engine" value="mysql" />
						</td>
					</tr>
					<tr class="cf7-datasource-database cf7-datasource-components">
						<th scope="row">'.esc_html( __('Hostname', 'cf7-datasource' ) ).'</th>
						<td>
							<input aria-label="'.esc_attr__('Hostname', 'cf7-datasource').'" type="text" name="cf7-database-hostname" />
						</td>
					</tr>
					<tr class="cf7-datasource-database cf7-datasource-components">
						<th scope="row">'.esc_html( __('Database name', 'cf7-datasource' ) ).'</th>
						<td>
							<input aria-label="'.esc_attr__('Database name', 'cf7-datasource').'" type="text" name="cf7-database-database" />
						</td>
					</tr>
					<tr class="cf7-datasource-database cf7-datasource-dns cf7-datasource-components">
						<th scope="row">'.esc_html( __('Database username', 'cf7-datasource' ) ).'</th>
						<td>
							<input aria-label="'.esc_attr__('Database username', 'cf7-datasource').'" type="text" name="cf7-database-username" />
						</td>
					</tr>
					<tr class="cf7-datasource-database cf7-datasource-dns cf7-datasource-components">
						<th scope="row">'.esc_html( __('Database password', 'cf7-datasource' ) ).'</th>
						<td>
							<input aria-label="'.esc_attr__('Database password', 'cf7-datasource').'" type="text" name="cf7-database-password" />
						</td>
					</tr>
					<tr class="cf7-datasource-database">
						<th scope="row">'.esc_html( __('Query', 'cf7-datasource' ) ).'</th>
						<td>
							<input aria-label="'.esc_attr__('Query', 'cf7-datasource').'" type="text" name="cf7-database-query" />
							<br><i>Ex. SELECT ID,post_title FROM {wpdb.posts} WHERE post_status=\'publish\'</i>
						</td>
					</tr>
					<tr class="cf7-datasource-database">
						<th scope="row"></th>
						<td style="text-align:right;">
							<a href="https://cf7-datasource.dwbooster.com/documentation#database" target="_blank" style="font-weight:bold;display:inline-block;padding:5px;border:2px solid #2271b1; color:#2271b1;text-decoration:none;">'.__('Data surce HELP', 'cf7-datasource').'</a>
						</td>
					</tr>';
		} // End ds_settings

		public function post_ds($settings, $data)
		{
			global $wpdb;

			$results = array();

			if(!empty($settings['attributes']))
			{
				// Replace parameters on conditions
				$condition = $settings['condition'];
				$condition = $this->replace_vars($condition, $data);
				$condition = $this->replace_constants($condition);
				$condition = str_replace(array('&&', '||'), array(' AND ', ' OR '), $condition);

				$query = 'SELECT '.$settings['attributes'].' FROM '.$wpdb->posts.' WHERE 1=1'.(!empty($condition) ? ' AND '.$condition : '');

				$results = $wpdb->get_results( $query, ARRAY_A );
				if($wpdb->last_error) $this->error_log($wpdb->last_error);
			}
			else $this->error_log('The attributes are required');
			return $results;
		} // End post_ds

		public function taxonomy_ds($settings, $data)
		{
			global $wpdb;

			$results = array();

			if(!empty($settings['attributes']))
			{
				if(!empty($settings['taxonomy']))
				{
					// Replace parameters on conditions
					$condition = $settings['condition'];
					$condition .= (!empty($condition) ? ' AND ' : '').'taxonomy="'.esc_sql($settings['taxonomy']).'"';
					$condition = $this->replace_vars($condition, $data);
					$condition = $this->replace_constants($condition);
					$condition = str_replace(array('&&', '||'), array(' AND ', ' OR '), $condition);
					$condition = preg_replace('/\b(term_id|slug|name)\b/i', "terms.$1", $condition);

					$attributes = $settings['attributes'];
					$attributes = preg_replace('/\b(term_id|slug|name)\b/i', "terms.$1", $attributes);

					$query = 'SELECT '.$attributes.' FROM '.$wpdb->terms.' terms JOIN '.$wpdb->term_taxonomy.' term_taxonomy ON (term_taxonomy.term_id=terms.term_id) WHERE 1=1'.(!empty($condition) ? ' AND '.$condition : '');
					$results = $wpdb->get_results( $query, ARRAY_A );
					if($wpdb->last_error) $this->error_log($wpdb->last_error);
				}
				else $this->error_log('The taxonomy is required');
			}
			else $this->error_log('The attributes are required');
			return $results;
		} // End taxonomy_ds

		public function user_ds($settings, $data)
		{
			$results = array();
			$users = array();
			if(!empty($settings['attributes']))
			{
				if(
					(is_numeric($settings['logged']) && $settings['logged'] == 1) ||
					(is_string($settings['logged']) && $settings['logged'] == true) ||
					(is_bool($settings['logged']) && $settings['logged'] == 'true')
				)
				{
					$users[] = wp_get_current_user();
				}
				else
				{
					global $wpdb;
					$query = 'SELECT ID FROM '.$wpdb->users.' WHERE 1=1 '.(!empty($settings['condition']) ? ' AND '.$settings['condition'] : '');
					$query = $this->replace_constants($query);
					$rows = $wpdb->get_results($query);
					$user = array();
					foreach($rows as $row)
					{
						$users[] = get_userdata($row->ID);
					}
				}

				if(!empty($users))
				{
					$attributes = explode(',', $settings['attributes']);
					foreach($users as $user)
					{
						$item = array();
						foreach($attributes as $attribute)
						{
							$attribute = trim($attribute);
							if(empty($attribute)) continue;
							$item[$attribute] = !empty($user->{$attribute}) ? $user->{$attribute} : '';
						}
						$results[] = $item;
					}
				}
				else $this->error_log('Not user');
			}
			else $this->error_log('The attributes are required');
			return $results;
		} // End user_ds

        public function database_ds($settings, $data)
		{
			global $wpdb;

			$query = '';
			$results = array();

			// Replace parameters on query
			$query = $this->replace_vars( $settings['query'], $data);
			$query = $this->replace_constants( $query);
			if(!empty($query))
			{
				if(
					!empty($settings['dns']) ||
					!empty($settings['hostname'])
				)
				{
					if(!empty($settings['dns']))
					{
						$dns = $settings['dns'];
					}
					else
					{
						$settings['engine'] = strtolower($settings['engine']);
						switch($settings['engine'])
						{
							case 'sqlite' :
								$dns = $settings['engine'].':'.$settings[ 'hostname' ];
							break;
							case 'firebird':
								$dns = $settings['engine'].':dbname='.$settings[ 'database' ];
							break;
							case 'ibm':
								$dns = $settings['engine'].':DRIVER={IBM DB2 ODBC DRIVER};hostnameNAME='.$settings[ 'hostname' ].';PROTOCOL=TCPIP;DATABASE='.$settings[ 'database' ];
							break;
							case 'informix':
								$dns = $settings['engine'].':hostname='.$settings[ 'hostname' ].';database='.$settings[ 'database' ];
							break;
							case 'sqlsrv':
								$dns = $settings['engine'].':Server='.$settings[ 'hostname' ].';Database='.$settings[ 'database' ];
							break;
							case 'oci':
								$dns = $settings['engine'].':dbname='.$settings[ 'database' ];
							break;
							default:
								$dns = $settings['engine'].':hostname='.$settings[ 'hostname' ].';dbname='.$settings[ 'database' ];
							break;
						}
					}

					try
					{
						if(!empty($dns))
						{
							$db_connect = new PDO( $dns, $settings[ 'username' ], $settings[ 'password' ], array(PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION));
						}

						$result = $db_connect->query($query);
						while($row = $result->fetch(PDO::FETCH_ASSOC))
						{
							foreach( $row as $_key => $_val )
							{
								if(!function_exists('mb_check_encoding') || true !== mb_check_encoding( $_val, 'UTF-8' ))
									$row[ $_key ] = utf8_encode($_val);
							}
							$results[] = (object)$row;
						}
					}
					catch(Exception $err)
					{
						$this->error_log($err->getMessage());
					}
				}
				else
				{
					$results = $wpdb->get_results( $query, ARRAY_A );
					if ($wpdb->last_error) $this->error_log($wpdb->last_error);
				}
			}
			else $this->error_log('The query is empty');
			return $results;
		} // End database_ds
	} // End class CF7_DATASOURCE_BASIC
}